@@ -4,12 +4,11 @@ import datetime
 from south.db import db
 from south.v2 import SchemaMigration
 from django.db import models
-from django.db.utils import DatabaseError
 
 def ignore_exists(fun, *args, **kwargs):
     try:
         fun(*args, **kwargs)
-    except DatabaseError, exc:
+    except Exception, exc:
         if "exists" in str(exc):
             # don't panic, everything is ok: it's just a hack
             if db.has_ddl_transactions:
