@@ -1,45 +1,42 @@
 from __future__ import absolute_import
 
-from datetime import timedelta
+from datetime import datetime, timedelta
 from time import time, mktime
 
-import django
-
-from django.conf import settings
 from django.core.exceptions import MultipleObjectsReturned, ValidationError
 from django.db import models
 from django.db.models import signals
 from django.utils.translation import ugettext_lazy as _
 
-from picklefield.fields import PickledObjectField
-
 from celery import schedules
 from celery import states
+from celery.events.state import heartbeat_expires
 from celery.utils.timeutils import timedelta_seconds
 
 from . import managers
+from .picklefield import PickledObjectField
 from .utils import now
 
-HEARTBEAT_EXPIRE = 150      # 2 minutes, 30 seconds
 TASK_STATE_CHOICES = zip(states.ALL_STATES, states.ALL_STATES)
 
 
 class TaskMeta(models.Model):
     """Task result/status."""
     task_id = models.CharField(_(u"task id"), max_length=255, unique=True)
-    status = models.CharField(_(u"task status"), max_length=50,
+    status = models.CharField(_(u"state"), max_length=50,
             default=states.PENDING, choices=TASK_STATE_CHOICES)
-    result = PickledObjectField(null=True, default=None)
+    result = PickledObjectField(null=True, default=None, editable=False)
     date_done = models.DateTimeField(_(u"done at"), auto_now=True)
     traceback = models.TextField(_(u"traceback"), blank=True, null=True)
     hidden = models.BooleanField(editable=False, default=False, db_index=True)
+    meta = PickledObjectField(_(u"meta"), null=True, default=None,
+                              editable=False)
 
     objects = managers.TaskManager()
 
     class Meta:
-        """Model meta-data."""
-        verbose_name = _(u"task meta")
-        verbose_name_plural = _(u"task meta")
+        verbose_name = _(u"task state")
+        verbose_name_plural = _(u"task states")
         db_table = "celery_taskmeta"
 
     def to_dict(self):
@@ -47,25 +44,26 @@ class TaskMeta(models.Model):
                 "status": self.status,
                 "result": self.result,
                 "date_done": self.date_done,
-                "traceback": self.traceback}
+                "traceback": self.traceback,
+                "children": (self.meta or {}).get("children")}
 
     def __unicode__(self):
-        return u"<Task: %s state->%s>" % (self.task_id, self.status)
+        return u"<Task: %s state=%s>" % (self.task_id, self.status)
 
 
 class TaskSetMeta(models.Model):
     """TaskSet result"""
-    taskset_id = models.CharField(_(u"task id"), max_length=255, unique=True)
+    taskset_id = models.CharField(_(u"group id"), max_length=255, unique=True)
     result = PickledObjectField()
-    date_done = models.DateTimeField(_(u"done at"), auto_now=True)
+    date_done = models.DateTimeField(_(u"created at"), auto_now=True)
     hidden = models.BooleanField(editable=False, default=False, db_index=True)
 
     objects = managers.TaskSetManager()
 
     class Meta:
         """Model meta-data."""
-        verbose_name = _(u"taskset meta")
-        verbose_name_plural = _(u"taskset meta")
+        verbose_name = _(u"saved group result")
+        verbose_name_plural = _(u"saved group results")
         db_table = "celery_tasksetmeta"
 
     def to_dict(self):
@@ -125,6 +123,12 @@ class CrontabSchedule(models.Model):
     day_of_week = models.CharField(_(u"day of week"),
                         max_length=64,
                         default="*")
+    day_of_month = models.CharField(_(u"day of month"),
+                        max_length=64,
+                        default="*")
+    month_of_year = models.CharField(_(u"month of year"),
+                        max_length=64,
+                        default="*")
 
     class Meta:
         verbose_name = _(u"crontab")
@@ -132,22 +136,35 @@ class CrontabSchedule(models.Model):
 
     def __unicode__(self):
         rfield = lambda f: f and str(f).replace(" ", "") or "*"
-        return u"%s %s %s (m/h/d)" % (rfield(self.minute),
-                                      rfield(self.hour),
-                                      rfield(self.day_of_week))
+        return u"%s %s %s %s %s (m/h/d/dM/MY)" % (rfield(self.minute),
+                                                  rfield(self.hour),
+                                                  rfield(self.day_of_week),
+                                                  rfield(self.day_of_month),
+                                                  rfield(self.month_of_year))
 
     @property
     def schedule(self):
         return schedules.crontab(minute=self.minute,
                                 hour=self.hour,
                                 day_of_week=self.day_of_week,
+                                day_of_month=self.day_of_month,
+                                month_of_year=self.month_of_year,
                                 nowfun=now)
 
     @classmethod
     def from_schedule(cls, schedule):
-        return cls(minute=schedule._orig_minute,
-                   hour=schedule._orig_hour,
-                   day_of_week=schedule._orig_day_of_week)
+        spec = {'minute': schedule._orig_minute,
+                'hour': schedule._orig_hour,
+                'day_of_week': schedule._orig_day_of_week,
+                'day_of_month': schedule._orig_day_of_month,
+                'month_of_year': schedule._orig_month_of_year}
+        try:
+            return cls.objects.get(**spec)
+        except cls.DoesNotExist:
+            return cls(**spec)
+        except MultipleObjectsReturned:
+            cls.objects.filter(**spec).delete()
+            return cls(**spec)
 
 
 class PeriodicTasks(models.Model):
@@ -225,6 +242,8 @@ class PeriodicTask(models.Model):
         self.exchange = self.exchange or None
         self.routing_key = self.routing_key or None
         self.queue = self.queue or None
+        if not self.enabled:
+            self.last_run_at = None
         super(PeriodicTask, self).save(*args, **kwargs)
 
     def __unicode__(self):
@@ -267,7 +286,7 @@ class WorkerState(models.Model):
 
     def is_alive(self):
         if self.last_heartbeat:
-            return time() < self.heartbeat_timestamp + HEARTBEAT_EXPIRE
+            return time() < heartbeat_expires(self.heartbeat_timestamp)
         return False
 
     @property
@@ -307,6 +326,11 @@ class TaskState(models.Model):
         get_latest_by = "tstamp"
         ordering = ["-tstamp"]
 
+    def save(self, *args, **kwargs):
+        if self.eta is not None:
+            self.eta = datetime.utcfromtimestamp(mktime(self.eta.timetuple()))
+        super(TaskState, self).save(*args, **kwargs)
+
     def __unicode__(self):
         name = self.name or "UNKNOWN"
         s = u"%s %s %s" % (self.state.ljust(10),
@@ -321,11 +345,3 @@ class TaskState(models.Model):
                                                  self.name or "UNKNOWN",
                                                  self.task_id,
                                                  self.tstamp)
-
-if (django.VERSION[0], django.VERSION[1]) >= (1, 1):
-    # Keep models away from syncdb/reset if Django database
-    # backend is not being used.
-    if "database" not in (getattr(settings,
-                            "CELERY_RESULT_BACKEND", None) or "database"):
-        for result_model in (TaskMeta, TaskSetMeta):
-            result_model._meta.managed = False
