@@ -63,21 +63,29 @@ class Camera(Polaroid):
         return obj
 
     def handle_task(self, (uuid, task), worker=None):
+        """Handle snapshotted event."""
         if task.worker and task.worker.hostname:
             worker = self.handle_worker((task.worker.hostname, task.worker))
-        return self.update_task(task.state, task_id=uuid,
-                defaults={"name": task.name,
-                          "args": task.args,
-                          "kwargs": task.kwargs,
-                          "eta": maybe_iso8601(task.eta),
-                          "expires": maybe_iso8601(task.expires),
-                          "state": task.state,
-                          "tstamp": make_aware(datetime.fromtimestamp(
-                                        task.timestamp)),
-                          "result": task.result or task.exception,
-                          "traceback": task.traceback,
-                          "runtime": task.runtime,
-                          "worker": worker})
+
+        NOT_SAVED_ATTRIBUTES = ('name', 'args', 'kwargs', 'eta')
+        defaults = {"name": task.name,
+                "args": task.args,
+                "kwargs": task.kwargs,
+                "eta": maybe_iso8601(task.eta),
+                "expires": maybe_iso8601(task.expires),
+                "state": task.state,
+                "tstamp": make_aware(datetime.fromtimestamp(
+                                     task.timestamp)),
+                "result": task.result or task.exception,
+                "traceback": task.traceback,
+                "runtime": task.runtime,
+                "worker": worker}
+        # If RECEIVED event is lost, some data is lost as it is stored
+        # only in RECEIVED event for efficiency. In this case we just
+        # do not overwrite this fields in TaskState instance row
+        [defaults.pop(attr) for attr in NOT_SAVED_ATTRIBUTES
+                                                 if defaults[attr] is None]
+        return self.update_task(task.state, task_id=uuid, defaults=defaults)
 
     def update_task(self, state, **kwargs):
         objects = self.TaskState.objects
